# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: swanlab/proto/core/collector/v1/collector_service.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'swanlab/proto/core/collector/v1/collector_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from swanlab.proto.common.v1 import common_pb2 as swanlab_dot_proto_dot_common_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7swanlab/proto/core/collector/v1/collector_service.proto\x12\x1fswanlab.proto.core.collector.v1\x1a$swanlab/proto/common/v1/common.proto\"M\n\x16\x43ollectorUploadRequest\x12\x33\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32%.swanlab.proto.common.v1.KeyValueList\";\n\x17\x43ollectorUploadResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t2\x88\x01\n\tCollector\x12{\n\x06Upload\x12\x37.swanlab.proto.core.collector.v1.CollectorUploadRequest\x1a\x38.swanlab.proto.core.collector.v1.CollectorUploadResponseB\rZ\x0b\x63ore/pkg/pbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'swanlab.proto.core.collector.v1.collector_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\013core/pkg/pb'
  _globals['_COLLECTORUPLOADREQUEST']._serialized_start=130
  _globals['_COLLECTORUPLOADREQUEST']._serialized_end=207
  _globals['_COLLECTORUPLOADRESPONSE']._serialized_start=209
  _globals['_COLLECTORUPLOADRESPONSE']._serialized_end=268
  _globals['_COLLECTOR']._serialized_start=271
  _globals['_COLLECTOR']._serialized_end=407
# @@protoc_insertion_point(module_scope)
